@@ -1,16 +1,25 @@
 /*
- * Copyright (c) 2007-2009 Code Synthesis Tools CC.
+ * Copyright (c) 2007-2011 Code Synthesis Tools CC.
  * Copyright (c) 2004 by Tim Bray and Sun Microsystems.
  *
  * For copying permission, see the accompanying COPYING file.
  */
 
+#include <xsde/c/pre.h>
+
 #define GENX_VERSION "cs-1"
 
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
-#include "genx.h"
+#include <xsde/config.h>
+
+#include <xsde/c/genx/genx.h>
+
+#ifdef XSDE_CUSTOM_ALLOCATOR
+#  include <xsde/allocator.h>
+#endif
 
 #define Boolean int
 #define True 1
@@ -105,11 +114,10 @@ struct genxAttribute_rec
  */
 struct genxWriter_rec
 {
-  FILE *       	  	   file;
   genxSender *    	   sender;
   genxStatus   	  	   status;
   writerSequence  	   sequence;
-  char            	   xmlChars[0x10000];
+  char            	   xmlChars[GENX_CHAR_TABLE_SIZE];
   void *          	   userData;
   int             	   nextPrefix;
   utf8                     empty;
@@ -126,6 +134,11 @@ struct genxWriter_rec
   char *                   etext[100];
   void *       		(* alloc)(void * userData, int bytes);
   void         		(* dealloc)(void * userData, void * data);
+
+  /* Pretty-printing state */
+  int                      ppIndent;
+  int                      ppDepth;
+  Boolean                  ppSimple;
 };
 
 /*******************************
@@ -151,7 +164,11 @@ static void * allocate(genxWriter w, int bytes)
   if (w->alloc)
     return (void *) (*w->alloc)(w->userData, bytes);
   else
+#ifdef XSDE_CUSTOM_ALLOCATOR
+    return (void *) xsde_alloc(bytes);
+#else
     return (void *) malloc(bytes);
+#endif
 }
 
 static void deallocate(genxWriter w, void * data)
@@ -159,7 +176,11 @@ static void deallocate(genxWriter w, void * data)
   if (w->dealloc)
     (*w->dealloc)(w->userData, data);
   else if (w->alloc == NULL)
+#ifdef XSDE_CUSTOM_ALLOCATOR
+    xsde_free(data);
+#else
     free(data);
+#endif
 }
 
 static utf8 copy(genxWriter w, constUtf8 from)
@@ -483,7 +504,7 @@ static Boolean isXMLChar(genxWriter w, int c)
 {
   if (c < 0)
     return False;
-  else if (c < 0x10000)
+  else if (c < GENX_CHAR_TABLE_SIZE)
     return (int) w->xmlChars[c];
   else
     return (c <= 0x10ffff);
@@ -494,7 +515,13 @@ static Boolean isLetter(genxWriter w, int c)
   if (c < 0 || c > 0xffff)
     return False;
   else
+  {
+#if GENX_CHAR_TABLE_SIZE == 0x10000
     return w->xmlChars[c] & GENX_LETTER;
+#else
+    return c < GENX_CHAR_TABLE_SIZE ? (w->xmlChars[c] & GENX_LETTER) : True;
+#endif
+  }
 }
 
 static Boolean isNameChar(genxWriter w, int c)
@@ -502,7 +529,13 @@ static Boolean isNameChar(genxWriter w, int c)
   if (c < 0 || c > 0xffff)
     return False;
   else
+  {
+#if GENX_CHAR_TABLE_SIZE == 0x10000
     return w->xmlChars[c] & GENX_NAMECHAR;
+#else
+    return c < GENX_CHAR_TABLE_SIZE ? (w->xmlChars[c] & GENX_NAMECHAR) : True;
+#endif
+  }
 }
 
 /*******************************
@@ -522,7 +555,11 @@ genxWriter genxNew(void * (* alloc)(void * userData, int bytes),
   if (alloc)
     w = (genxWriter) (*alloc)(userData, sizeof(struct genxWriter_rec));
   else
+#ifdef XSDE_CUSTOM_ALLOCATOR
+    w = (genxWriter) xsde_alloc(sizeof(struct genxWriter_rec));
+#else
     w = (genxWriter) malloc(sizeof(struct genxWriter_rec));
+#endif
 
   if (w == NULL)
     return NULL;
@@ -590,6 +627,8 @@ genxWriter genxNew(void * (* alloc)(void * userData, int bytes),
   xml->declCount = 1;
   xml->declaration = xml->defaultDecl;
 
+  w->ppIndent = 0; /* Pretty-printing is disabled by default. */
+
   return w;
 }
 
@@ -629,6 +668,24 @@ void * genxGetUserData(genxWriter w)
   return w->userData;
 }
 
+/*
+ * get/set pretty-printing
+ */
+genxStatus genxSetPrettyPrint(genxWriter w, int ind)
+{
+  if (w->sequence == SEQUENCE_NO_DOC)
+    w->ppIndent = ind;
+  else
+    w->status = GENX_SEQUENCE_ERROR;
+
+  return w->status;
+}
+
+int genxGetPrettyPrint(genxWriter w)
+{
+  return w->ppIndent;
+}
+
 /*
  * get/set allocator
  */
@@ -1103,12 +1160,7 @@ static genxStatus sendx(genxWriter w, constUtf8 s)
   if (w->sender)
     return (*w->sender->send)(w->userData, s);
   else
-  {
-    if (fputs((const char *) s, w->file) == -1)
-      return GENX_IO_ERROR;
-    else
-      return GENX_SUCCESS;
-  }
+    return GENX_IO_ERROR;
 }
 
 static genxStatus sendxBounded(genxWriter w, constUtf8 start, constUtf8 end)
@@ -1116,11 +1168,7 @@ static genxStatus sendxBounded(genxWriter w, constUtf8 start, constUtf8 end)
   if (w->sender)
     return (*w->sender->sendBounded)(w->userData, start, end);
   else
-    /* Looks like on VxWorks fwrite returns a signed type which causes warnings. */
-    if ((unsigned) fwrite(start, 1, end - start, w->file) != (unsigned) (end - start))
-      return GENX_IO_ERROR;
-    else
-      return GENX_SUCCESS;
+    return GENX_IO_ERROR;
 }
 
 #define SendCheck(w,s) if ((w->status=sendx(w,(utf8)s))!=GENX_SUCCESS) return w->status;
@@ -1131,31 +1179,38 @@ static genxStatus sendxBounded(genxWriter w, constUtf8 start, constUtf8 end)
  *  for internal routines.
  */
 
-/*
- * Start a document
- */
-genxStatus genxStartDocFile(genxWriter w, FILE * file)
-{
-  if (w->sequence != SEQUENCE_NO_DOC)
-    return w->status = GENX_SEQUENCE_ERROR;
-
-  w->sequence = SEQUENCE_PRE_DOC;
-  w->file = file;
-  w->sender = NULL;
-  return GENX_SUCCESS;
-}
-
 genxStatus genxStartDocSender(genxWriter w, genxSender * sender)
 {
   if (w->sequence != SEQUENCE_NO_DOC)
     return w->status = GENX_SEQUENCE_ERROR;
 
   w->sequence = SEQUENCE_PRE_DOC;
-  w->file = NULL;
   w->sender = sender;
+
+  if (w->ppIndent)
+  {
+    w->ppSimple = True;
+    w->ppDepth = 0;
+  }
+
   return GENX_SUCCESS;
 }
 
+/*
+ * Output new line and indentation.
+ */
+static genxStatus writeIndentation(genxWriter w)
+{
+  int i, n;
+  SendCheck(w, "\n");
+  n = w->ppDepth * w->ppIndent;
+
+  for (i = 0; i < n; i++)
+    SendCheck(w, " ");
+
+  return w->status;
+}
+
 /*
  * Write out the attributes we've been gathering up for an element.  We save
  *  them until we've gathered them all so they can be writen in canonical
@@ -1181,6 +1236,16 @@ static genxStatus writeStartTag(genxWriter w)
     unsetDefaultNamespace(w);
   w->status = GENX_SUCCESS;
 
+  if (w->ppIndent)
+  {
+    if (w->ppDepth)
+      if (writeIndentation (w) != GENX_SUCCESS)
+        return w->status;
+
+    w->ppDepth++;
+    w->ppSimple = True;
+  }
+
   SendCheck(w, "<");
   if (e->ns && (e->ns->declaration != w->xmlnsEquals))
   {
@@ -1465,7 +1530,7 @@ static genxStatus collectAttributeValue (genxWriter w, collector* value,
 
   utf8 last = (utf8) start;
 
-  while (*start && (end == NULL || start < end))
+  while (end != NULL ? start < end : *start)
   {
     int c = genxNextUnicodeChar(&start);
 
@@ -1624,6 +1689,17 @@ genxStatus genxEndElement(genxWriter w)
     ;
   e = (genxElement) w->stack.pointers[--i];
 
+  if (w->ppIndent)
+  {
+    w->ppDepth--;
+
+    if (!w->ppSimple)
+      if (writeIndentation (w) != GENX_SUCCESS)
+        return w->status;
+
+    w->ppSimple = False;
+  }
+
   SendCheck(w, "</");
   if (e->ns && e->ns->declaration != w->xmlnsEquals)
   {
@@ -1922,11 +1998,8 @@ genxStatus genxEndDocument(genxWriter w)
   if (w->sequence != SEQUENCE_POST_DOC)
     return w->status = GENX_SEQUENCE_ERROR;
 
-  if (w->file)
-    fflush(w->file);
-  else
-    if ((w->status = (*w->sender->flush)(w->userData)) != GENX_SUCCESS)
-      return w->status;
+  if ((w->status = (*w->sender->flush)(w->userData)) != GENX_SUCCESS)
+    return w->status;
 
   w->sequence = SEQUENCE_NO_DOC;
   return GENX_SUCCESS;
@@ -2116,3 +2189,4 @@ char * genxGetVersion()
   return GENX_VERSION;
 }
 
+#include <xsde/c/post.h>
