@@ -1,10 +1,14 @@
 // file      : xsde/cxx/parser/non-validating/hex-binary.cxx
 // author    : Boris Kolpackov <boris@codesynthesis.com>
-// copyright : Copyright (c) 2005-2008 Code Synthesis Tools CC
+// copyright : Copyright (c) 2005-2011 Code Synthesis Tools CC
 // license   : GNU GPL v2 + exceptions; see accompanying LICENSE file
 
 #include <xsde/cxx/config.hxx>
 
+#ifdef XSDE_CUSTOM_ALLOCATOR
+#  include <xsde/cxx/allocator.hxx>
+#endif
+
 #include <xsde/cxx/parser/non-validating/hex-binary.hxx>
 
 static unsigned char
@@ -30,9 +34,78 @@ namespace xsde
     {
       namespace non_validating
       {
+        hex_binary_pimpl::
+        ~hex_binary_pimpl ()
+        {
+          if (!base_ && buf_)
+          {
+#ifndef XSDE_CUSTOM_ALLOCATOR
+            delete buf_;
+#else
+            buf_->~buffer ();
+            cxx::free (buf_);
+#endif
+          }
+        }
+
+        void hex_binary_pimpl::
+        _reset ()
+        {
+          hex_binary_pskel::_reset ();
+
+          if (!base_ && buf_)
+          {
+#ifndef XSDE_CUSTOM_ALLOCATOR
+            delete buf_;
+#else
+            buf_->~buffer ();
+            cxx::free (buf_);
+#endif
+            buf_ = 0;
+          }
+        }
+
+        hex_binary_pimpl::
+        hex_binary_pimpl (bool base)
+            : base_ (base), buf_ (0)
+        {
+        }
+
+        void hex_binary_pimpl::
+        pre_impl (buffer* b)
+        {
+          buf_ = b;
+        }
+
         void hex_binary_pimpl::
         _pre ()
         {
+          if (buf_ == 0)
+          {
+#ifndef XSDE_CUSTOM_ALLOCATOR
+            buf_ = new buffer ();
+#else
+            buf_ = static_cast<buffer*> (alloc (sizeof (buffer)));
+
+#ifdef XSDE_EXCEPTIONS
+            alloc_guard ag (buf_);
+            new (buf_) buffer ();
+            ag.release ();
+#else
+            if (buf_)
+              new (buf_) buffer ();
+#endif
+#endif
+
+#ifndef XSDE_EXCEPTIONS
+            if (buf_ == 0)
+            {
+              _sys_error (sys_error::no_memory);
+              return;
+            }
+#endif
+          }
+
 #ifdef XSDE_EXCEPTIONS
           str_.assign ("", 0);
 #else
@@ -69,18 +142,6 @@ namespace xsde
           }
         }
 
-        struct buffer_guard
-        {
-          ~buffer_guard () { delete p_; }
-          buffer_guard (buffer* p) : p_ (p) {}
-
-          void
-          release () { p_ = 0; }
-
-        private:
-          buffer* p_;
-        };
-
         buffer* hex_binary_pimpl::
         post_hex_binary ()
         {
@@ -88,21 +149,11 @@ namespace xsde
           ro_string::size_type size = trim_right (tmp);
 
           size_t n = size / 2;
-          buffer* buf = new buffer ();
-
-#ifndef XSDE_EXCEPTIONS
-          if (buf == 0)
-          {
-            _sys_error (sys_error::no_memory);
-            return 0;
-          }
-#endif
-          buffer_guard bufg (buf);
 
 #ifdef XSDE_EXCEPTIONS
-          buf->size (n);
+          buf_->size (n);
 #else
-          if (buf->size (n))
+          if (buf_->size (n))
           {
             _sys_error (sys_error::no_memory);
             return 0;
@@ -112,7 +163,7 @@ namespace xsde
           if (n != 0)
           {
             const char* src = tmp.data ();
-            char* dst = buf->data ();
+            char* dst = buf_->data ();
             size_t i = 0;
 
             for (; i < n; ++i)
@@ -127,8 +178,9 @@ namespace xsde
             }
           }
 
-          bufg.release ();
-          return buf;
+          buffer* r = buf_;
+          buf_ = 0;
+          return r;
         }
       }
     }
