@@ -1,11 +1,12 @@
 // file      : xsde/cxx/parser/non-validating/nmtoken.cxx
 // author    : Boris Kolpackov <boris@codesynthesis.com>
-// copyright : Copyright (c) 2005-2009 Code Synthesis Tools CC
+// copyright : Copyright (c) 2005-2011 Code Synthesis Tools CC
 // license   : GNU GPL v2 + exceptions; see accompanying LICENSE file
 
 #include <xsde/cxx/config.hxx>
 
 #include <xsde/cxx/parser/non-validating/nmtoken.hxx>
+#include <xsde/cxx/parser/non-validating/string-common.hxx>
 
 namespace xsde
 {
@@ -57,8 +58,7 @@ namespace xsde
         char* nmtoken_pimpl::
         post_nmtoken ()
         {
-          ro_string tmp (str_.data (), str_.size ());
-          str_.truncate (trim_right (tmp));
+          string_common::process_facets (str_, _facets ());
           return str_.detach ();
         }
       }
