@@ -1,11 +1,12 @@
 // file      : xsde/cxx/parser/validating/nmtoken-stl.cxx
 // author    : Boris Kolpackov <boris@codesynthesis.com>
-// copyright : Copyright (c) 2005-2008 Code Synthesis Tools CC
+// copyright : Copyright (c) 2005-2011 Code Synthesis Tools CC
 // license   : GNU GPL v2 + exceptions; see accompanying LICENSE file
 
 #include <xsde/cxx/xml/char-table.hxx>
 
 #include <xsde/cxx/parser/validating/nmtoken-stl.hxx>
+#include <xsde/cxx/parser/validating/string-common.hxx>
 
 namespace xsde
 {
@@ -38,8 +39,11 @@ namespace xsde
         void nmtoken_pimpl::
         _post ()
         {
-          ro_string tmp (str_);
-          ro_string::size_type size = trim_right (tmp);
+          if (!string_common::validate_facets (str_, _facets (), _context ()))
+            return;
+
+          typedef std::string::size_type size_type;
+          size_type size = str_.size ();
 
           // For now we are only checking the US-ASCII characters.
           //
@@ -48,7 +52,7 @@ namespace xsde
 
           if (ok)
           {
-            for (ro_string::size_type i = 0; i < size; ++i)
+            for (size_type i = 0; i < size; ++i)
             {
               unsigned char c = static_cast<unsigned char> (str_[i]);
 
@@ -60,8 +64,6 @@ namespace xsde
             }
           }
 
-          str_.resize (size);
-
           if (!ok)
             _schema_error (schema_error::invalid_nmtoken_value);
         }
