@@ -1,6 +1,6 @@
 // file      : xsde/cxx/parser/validating/nmtoken.cxx
 // author    : Boris Kolpackov <boris@codesynthesis.com>
-// copyright : Copyright (c) 2005-2008 Code Synthesis Tools CC
+// copyright : Copyright (c) 2005-2011 Code Synthesis Tools CC
 // license   : GNU GPL v2 + exceptions; see accompanying LICENSE file
 
 #include <xsde/cxx/config.hxx>
@@ -8,6 +8,7 @@
 #include <xsde/cxx/xml/char-table.hxx>
 
 #include <xsde/cxx/parser/validating/nmtoken.hxx>
+#include <xsde/cxx/parser/validating/string-common.hxx>
 
 namespace xsde
 {
@@ -59,8 +60,11 @@ namespace xsde
         void nmtoken_pimpl::
         _post ()
         {
-          ro_string tmp (str_.data (), str_.size ());
-          ro_string::size_type size = trim_right (tmp);
+          if (!string_common::validate_facets (str_, _facets (), _context ()))
+            return;
+
+          typedef string::size_type size_type;
+          size_type size = str_.size ();
 
           // For now we are only checking the US-ASCII characters.
           //
@@ -69,7 +73,7 @@ namespace xsde
 
           if (ok)
           {
-            for (ro_string::size_type i = 0; i < size; ++i)
+            for (size_type i = 0; i < size; ++i)
             {
               unsigned char c = static_cast<unsigned char> (str_[i]);
 
@@ -81,8 +85,6 @@ namespace xsde
             }
           }
 
-          str_.truncate (size);
-
           if (!ok)
             _schema_error (schema_error::invalid_nmtoken_value);
         }
