@@ -1,10 +1,14 @@
 // file      : xsde/cxx/parser/validating/nmtokens.cxx
 // author    : Boris Kolpackov <boris@codesynthesis.com>
-// copyright : Copyright (c) 2005-2008 Code Synthesis Tools CC
+// copyright : Copyright (c) 2005-2011 Code Synthesis Tools CC
 // license   : GNU GPL v2 + exceptions; see accompanying LICENSE file
 
 #include <xsde/cxx/config.hxx>
 
+#ifdef XSDE_CUSTOM_ALLOCATOR
+#  include <xsde/cxx/allocator.hxx>
+#endif
+
 #include <xsde/cxx/parser/validating/nmtokens.hxx>
 
 namespace xsde
@@ -18,28 +22,78 @@ namespace xsde
         nmtokens_pimpl::
         ~nmtokens_pimpl ()
         {
-          delete seq_;
+          if (!base_ && seq_)
+          {
+#ifndef XSDE_CUSTOM_ALLOCATOR
+            delete seq_;
+#else
+            seq_->~string_sequence ();
+            cxx::free (seq_);
+#endif
+          }
+        }
+
+        void nmtokens_pimpl::
+        _reset ()
+        {
+          nmtokens_pskel::_reset ();
+
+          if (!base_ && seq_)
+          {
+#ifndef XSDE_CUSTOM_ALLOCATOR
+            delete seq_;
+#else
+            seq_->~string_sequence ();
+            cxx::free (seq_);
+#endif
+            seq_ = 0;
+          }
+
+          parser_._reset ();
         }
 
         nmtokens_pimpl::
-        nmtokens_pimpl ()
-            : seq_ (0)
+        nmtokens_pimpl (bool base)
+            : base_ (base), seq_ (0)
         {
         }
 
+        void nmtokens_pimpl::
+        pre_impl (string_sequence* seq)
+        {
+          seq_ = seq;
+        }
+
         void nmtokens_pimpl::
         _pre ()
         {
-          delete seq_;
-          seq_ = new string_sequence ();
-
-#ifndef XSDE_EXCEPTIONS
           if (seq_ == 0)
           {
-            _sys_error (sys_error::no_memory);
-            return;
-          }
+#ifndef XSDE_CUSTOM_ALLOCATOR
+            seq_ = new string_sequence ();
+#else
+            seq_ = static_cast<string_sequence*> (
+              alloc (sizeof (string_sequence)));
+
+#ifdef XSDE_EXCEPTIONS
+            alloc_guard ag (seq_);
+            new (seq_) string_sequence ();
+            ag.release ();
+#else
+            if (seq_)
+              new (seq_) string_sequence ();
 #endif
+#endif
+
+#ifndef XSDE_EXCEPTIONS
+            if (seq_ == 0)
+            {
+              _sys_error (sys_error::no_memory);
+              return;
+            }
+#endif
+          }
+
           nmtokens_pskel::_pre ();
         }
 
@@ -87,29 +141,21 @@ namespace xsde
           {
             char* x = parser_.post_nmtoken ();
 
+            // push_back() frees the string if it fails.
+            //
 #ifndef XSDE_EXCEPTIONS
             if (!parser_._error_type ())
             {
-              if (seq_->push_back_attach (x))
-              {
-                delete[] x;
+              if (seq_->push_back (x))
                 _sys_error (sys_error::no_memory);
-              }
             }
             else
               parser_._copy_error (ctx);
 #else
-            seq_->push_back_attach (x);
+            seq_->push_back (x);
 #endif
           }
         }
-
-        void nmtokens_pimpl::
-        _reset ()
-        {
-          nmtokens_pskel::_reset ();
-          parser_._reset ();
-        }
       }
     }
   }
