@@ -1,9 +1,10 @@
 // file      : xsde/cxx/parser/validating/token-stl.cxx
 // author    : Boris Kolpackov <boris@codesynthesis.com>
-// copyright : Copyright (c) 2005-2008 Code Synthesis Tools CC
+// copyright : Copyright (c) 2005-2011 Code Synthesis Tools CC
 // license   : GNU GPL v2 + exceptions; see accompanying LICENSE file
 
 #include <xsde/cxx/parser/validating/token-stl.hxx>
+#include <xsde/cxx/parser/validating/string-common.hxx>
 
 namespace xsde
 {
@@ -33,36 +34,15 @@ namespace xsde
             str_ += s;
         }
 
+        void token_pimpl::
+        _post ()
+        {
+          string_common::validate_facets (str_, _facets (), _context ());
+        }
+
         std::string token_pimpl::
         post_token ()
         {
-          std::string::size_type size = str_.size ();
-          std::string::size_type j = 0;
-
-          bool subs = false;
-
-          for (std::string::size_type i = 0; i < size; ++i)
-          {
-            char c = str_[i];
-
-            if (c == 0x20 || c == 0x0A || c == 0x0D || c == 0x09)
-            {
-              subs = true;
-            }
-            else
-            {
-              if (subs)
-              {
-                subs = false;
-                str_[j++] = 0x20;
-              }
-
-              str_[j++] = c;
-            }
-          }
-
-          str_.resize (j);
-
           std::string r;
           r.swap (str_);
           return r;
