@@ -1,6 +1,6 @@
 // file      : xsde/cxx/serializer/non-validating/xml-schema-sskel.cxx
 // author    : Boris Kolpackov <boris@codesynthesis.com>
-// copyright : Copyright (c) 2005-2009 Code Synthesis Tools CC
+// copyright : Copyright (c) 2005-2011 Code Synthesis Tools CC
 // license   : GNU GPL v2 + exceptions; see accompanying LICENSE file
 
 #include <xsde/cxx/serializer/non-validating/xml-schema-sskel.hxx>
@@ -24,17 +24,6 @@ namespace xsde
 #endif
         }
 
-        // any_simple_type_sskel
-        //
-        void any_simple_type_sskel::
-        pre ()
-        {
-#ifdef XSDE_REUSE_STYLE_TIEIN
-          if (any_simple_type_impl_)
-            any_simple_type_impl_->pre ();
-#endif
-        }
-
         // static/dynamic_type function implementations.
         //
 #ifdef XSDE_POLYMORPHIC
