@@ -1,9 +1,14 @@
 // file      : xsde/cxx/serializer/validating/token.cxx
 // author    : Boris Kolpackov <boris@codesynthesis.com>
-// copyright : Copyright (c) 2005-2008 Code Synthesis Tools CC
+// copyright : Copyright (c) 2005-2011 Code Synthesis Tools CC
 // license   : GNU GPL v2 + exceptions; see accompanying LICENSE file
 
 #include <xsde/cxx/serializer/validating/token.hxx>
+#include <xsde/cxx/serializer/validating/string-common.hxx>
+
+#ifdef XSDE_CUSTOM_ALLOCATOR
+#  include <xsde/cxx/allocator.hxx>
+#endif
 
 namespace xsde
 {
@@ -16,8 +21,15 @@ namespace xsde
         token_simpl::
         ~token_simpl ()
         {
-          if (free_)
-            delete[] const_cast<char*> (value_);
+          if (free_ && value_)
+          {
+            char* v = const_cast<char*> (value_);
+#ifndef XSDE_CUSTOM_ALLOCATOR
+            delete[] v;
+#else
+            cxx::free (v);
+#endif
+          }
         }
 
         void token_simpl::
@@ -50,13 +62,24 @@ namespace xsde
             ok = false;
 
           if (ok)
-            _characters (value_);
+          {
+            if (string_common::validate_facets (
+                  value_, _facets (), _context ()))
+            {
+              _characters (value_);
+            }
+          }
           else
             _schema_error (schema_error::invalid_token_value);
 
           if (free_)
           {
-            delete[] const_cast<char*> (value_);
+            char* v = const_cast<char*> (value_);
+#ifndef XSDE_CUSTOM_ALLOCATOR
+            delete[] v;
+#else
+            cxx::free (v);
+#endif
             value_ = 0;
           }
         }
